# -*- coding: utf-8 -*-
'''
Created on 2nd November 2017
Python 2.7 (should be compatible with Python 3)

@author: Philippe Cayeux

'''
#
# Define a __version__ variable to track changes in the package.
#
__version__ = '1.3.4'     
import os
import gc

from flask import Flask
from flask_sslify import SSLify
from pymongo import MongoClient
import pandas
#
# Define DB and Flask application depending on context. The client to be passed to routes is created with connect=False and maxPoolSize=1 because pymongo is not fork-safe.
#
from config import config, context

db = MongoClient(config.DATABASE_URI, config.PORT, connect=False, maxPoolSize=1).__getattr__(config.DATABASE_NAME)

app = Flask(__name__)
app.config.from_object(config)
if not config.DEBUG and not config.TESTING:    # Enforce HTTPS
	_sslifyApp = SSLify(app)

# Startup operations.
# Dictionary of unpickled pickle files loaded at start to save time when processing requests.
if config.PRELOAD:
	# Preload dumps only for the current date in order to save memory.
	now = pandas.Timestamp.now()
	# clientTmp is used only for startup operations as it is used before forking and is not safe after forking.
	clientTmp = MongoClient(app.config.get('DATABASE_URI'), app.config.get('PORT'))
	dbTmp = clientTmp.__getattr__(app.config.get('DATABASE_NAME'))  # Used only for preloading
	milestone = (0, 12)
	availableDates = sorted([pandas.Timestamp(date) for date in dbTmp['classifiers'].find({}, projection=['refDate']).distinct('refDate')])
	currentDate = pandas.Timestamp.now().normalize()
	validDates = [date for date in availableDates if date+pandas.offsets.MonthOffset(months=milestone[1]) <= currentDate]
	clfDate = validDates[-1]
	print("Loading estimators for %s in RAM."%clfDate.strftime('%Y-%m-%d'))
	from OnLineEstimators.OnlineOperations.estimatorsManagement import loadClassifiersFromDB          # Import here and not at beginning to avoid circular imports as this packages imports db 
	app._estimatorsDict = loadClassifiersFromDB(basenames=['valid'] if config.TESTING else None, \
											    datesSequence=[clfDate.strftime('%Y%m%d')] if not config.TESTING else None, nbMax=20)  # Set as an extra attribute of the Flask object to allow context overriding for tests
	clientTmp.close()
	print("Loaded in %d s."%((pandas.Timestamp.now()-now).seconds))
else:
	app._estimatorsDict = {}
gc.collect()

from api import views

if __name__ == '__main__':
	host = '127.0.0.1' if app.config["DEBUG"] else '0.0.0.0'
	port = int(os.environ.get('PORT', 5000))
	app.run(host=host, port=port, debug=app.config["DEBUG"])
