# -*- coding: utf-8 -*-
'''
Created on 2nd November 2017
Python 2.7 (should be compatible with Python 3)

@author: Philippe Cayeux

This module defines endpoints.
'''

from past.builtins import basestring
import os
import re
import decimal
import warnings
from collections import OrderedDict

import numpy
import pandas
from flask import jsonify, request

from api import app, context
#
# Define a route for health check.
#
@app.route('/status', methods=['GET'])
def getHealthCheck():
	'''
	This function sends a health status (status 200 after a dummy request to DB).
	'''
	try:
		from api import db
		_ = db.SDdata.count()
		response = jsonify({})
	except Exception as error:
		response = jsonify({'error': str(error.args)})
		response.status_code = 500
	return response
#
# Error management
#
@app.errorhandler(Exception)
def manageExceptions(error):
	# Make a response with the error (and remove debug information as far as possible). Encapsulate the processing in a try/except statement to be sure that the exception catching does not raise
	# another exception!
	try:
		errorMessage = error.args[0] if len(error.args) > 0 and isinstance(error.args[0], basestring) else str(error.args)
		response = jsonify({'error': errorMessage})
		response.status_code = 200
	except:
		errorMessage = "Internal error"
		response = jsonify({'error': errorMessage})
		response.status_code = 200
	return response
#
@app.errorhandler(404)
def notFound(_error):
	response = jsonify({'error': '404 - Not Found: ' + request.path})
	response.status_code = 404
	return response
